package minutes;

import com.meterware.httpunit.*;

import java.io.*;
import java.util.regex.*;
import java.text.*;
import java.util.*;
import java.net.*;

public class MinutesFetcher
{
    static final String notnumber = "[^0-9]*";
    static final String groupnumber = "([0-9]+)";
    static final String groupmonth = "(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)[A-Z]*";

    public static void main(String args[])
    {
	if (args.length!=1) 
	    {
		System.out.println("Usage: <minutesfetcher> configfile");
		System.exit(0);
	    }

	try {
	    SimpleConfigFile config = new SimpleConfigFile(args[0]);

	    MinutesFetcher mf = new MinutesFetcher(config.getString("PHONE"), 
						   config.getString("PASSWORD"));
	    String text;

	    if (true)
		text = mf.fetch();
	    else
		text = mf.load();

	    mf.process(text);

	    FileWriter fw = new FileWriter("/var/tmp/sprintpcs/eolsontoday.txt");
	    fw.write(mf.phpArray.serialize()+"\n");
	    fw.close();

	    fw = new FileWriter("/var/tmp/sprintpcs/eolsonhistory.txt", true);
	    fw.write(mf.phpArray.serialize()+"\n");
	    fw.close();
	} catch (Exception ex) {
	    System.out.println("Exception : "+ex);
	}
    }

    public class WCL implements WebClientListener
    {
	public void requestSent(WebClient src, WebRequest req)
	{
	    try {
		System.out.println("Sent: "+req.getMethod()+" "+req.getURL());
	    } catch (Exception ex) {
		System.out.println("exception: "+ex);
	    }
	}

	public void responseReceived(WebClient src, WebResponse resp)
	{
	    System.out.println("Recv");
	}

    }

    PhpArray phpArray;
    PhpArray minutes;
    String userid;
    String password;

    public MinutesFetcher(String userid, String password)
    {
	this.userid = userid;
	this.password = password;

	phpArray = new PhpArray();
	minutes = new PhpArray();
	phpArray.put("minutes", minutes);
    }

    public String load() throws Exception
    {
	BufferedReader ins = new BufferedReader(new FileReader("/tmp/minutes.log"));
	StringBuffer sb = new StringBuffer();

	String line;

	while ((line = ins.readLine())!=null)
	    {
		sb.append(line+"\n");
	    }

	return sb.toString();
    }

    public String fetch() throws Exception
    {
	HttpUnitOptions.setExceptionsThrownOnScriptError(false);
	HttpUnitOptions.setScriptingEnabled(false);

	WebConversation wc = new WebConversation();
	wc.addClientListener(new WCL());
	ClientProperties cp = wc.getClientProperties();
	cp.setUserAgent("Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)");

	WebResponse resp = wc.getResponse("http://www.sprint.com/personal/wireless/manage.html");

	WebForm forms[] = resp.getForms();

	// Find the right log-in form.
	int goodform = -1;
	for (int formnum = 0; formnum < forms.length; formnum++)
	    {
		WebForm form = forms[formnum];

		if (form.isTextParameter("userid") && 
		    form.isTextParameter("password"))
		    goodform = formnum;
	    }

	if (goodform<0)
	    {
		System.out.println("Couldn't find the right form");
		return null;
	    }

	WebForm form = forms[goodform];

	// submit the form
	form.setParameter("userid", userid);
	form.setParameter("password", password);
	resp = form.submit();

	// Follow the link which leads to the usage page
	WebLink link = resp.getLinkWith("Details...");
	resp = link.click();

	// here's our text. Now we harvest it.
	String text = resp.getText();

	FileWriter fw = new FileWriter("/tmp/minutes.log");
	fw.write(text);
	fw.close();

	return text;
    }

    public void process(String text)
    {
	String[] fields = new String[] {"MINUTES IN PLAN", 
					"OFF-PEAK-OPTION", 
					"WIRELESS WEB MESSAGES"};

	for(String field: fields)
	    {
		searchForUsage(text, field);
	    }

	getCurrentAsOfTime(text);
	getStartEndTime(text);

	phpArray.put("updatetime", System.currentTimeMillis()/1000);

	System.out.println(phpArray.serialize());

    }

    public void getCurrentAsOfTime(String text)
    {
	String re = "Current as of.*?"+groupmonth+notnumber+groupnumber+notnumber+groupnumber+
	    notnumber+groupnumber+notnumber+groupnumber+"\\s"+"(AM|PM)";
	
	Pattern pat = Pattern.compile(re, Pattern.MULTILINE | Pattern.DOTALL);
	Matcher mat = pat.matcher(text);
	boolean res = mat.find();

	if (res)
	    {
		String month = mat.group(1);
		int dayOfMonth = Integer.parseInt(mat.group(2));
		int year = Integer.parseInt(mat.group(3));
		int hour = Integer.parseInt(mat.group(4));
		int minute = Integer.parseInt(mat.group(5));
		String ampm = mat.group(6);

		SimpleDateFormat sdf = new SimpleDateFormat("MMM dd yyyy h:m a");
		Date date = sdf.parse(month+" "+dayOfMonth+" "+year+" "+hour+":"+minute+" "+ampm, new ParsePosition(0));
		long time = date.getTime()/1000;

		System.out.println("Current as of time: "+date);
		phpArray.put("currentasoftime", time);
	    }
    }
    
    public void getStartEndTime(String text)
    {
	String re = "Invoice period.*?"+groupmonth+notnumber+groupnumber+notnumber+groupnumber +
	    ".*?"+groupmonth+notnumber+groupnumber+notnumber+groupnumber;
	
	Pattern pat = Pattern.compile(re, Pattern.MULTILINE | Pattern.DOTALL);
	Matcher mat = pat.matcher(text);
	boolean res = mat.find();

	if (res)
	    {
		SimpleDateFormat sdf = new SimpleDateFormat("MMM dd yyyy");

		String month = mat.group(1);
		int dayOfMonth = Integer.parseInt(mat.group(2));
		int year = Integer.parseInt(mat.group(3));

		Date startdate = sdf.parse(month+" "+dayOfMonth+" "+year, new ParsePosition(0));
		long starttime = startdate.getTime()/1000;

		month = mat.group(4);
		dayOfMonth = Integer.parseInt(mat.group(5));
		year = Integer.parseInt(mat.group(6));

		Date enddate = sdf.parse(month+" "+dayOfMonth+" "+year, new ParsePosition(0));
		long endtime = enddate.getTime()/1000;

		System.out.println("Billing cycle: "+startdate+" - "+enddate);

		phpArray.put("starttime", starttime);
		phpArray.put("endtime", endtime);
	    }

    }

    public void searchForUsage(String text, String fieldname)
    {
	String re = fieldname + notnumber + groupnumber + notnumber + groupnumber + notnumber + groupnumber + ".*";

	Pattern pat = Pattern.compile(re, Pattern.MULTILINE | Pattern.DOTALL);
	Matcher mat = pat.matcher(text);
	boolean res = mat.find();

	if (!res)
	    return;

	int groupCount = mat.groupCount();

	int totalMinutes = Integer.parseInt(mat.group(1));
	int usedMinutes = Integer.parseInt(mat.group(2));
	int remainingMinutes = Integer.parseInt(mat.group(3));

	PhpArray mintypes = new PhpArray();
	mintypes.put("plan", totalMinutes);
	mintypes.put("used", usedMinutes);

	minutes.put(fieldname, mintypes);

	System.out.println(fieldname+" "+usedMinutes+"/"+totalMinutes);

	return;
    }

    public static void dumpCookies(WebClient wc)
    {
	String[] cookienames = wc.getCookieNames();

	for (String cookiename : cookienames)
	    {
		System.out.println(cookiename+" -> "+wc.getCookieValue(cookiename));
	    }
    }
}
